/******
 * XMasTree89
 * ARCH : STC15W408AS-DIP16 / 8051 / 89C5x / STC32G12K128 / ATMEGA328P
 * Cyril BARBATO 2024
 * For Ardui89 UNO / Ardui89 MINI / Ardui89 Cinque Uno or Arduino UNO
 */

#include "pitches.h"

//#define DEBUG 1


/*******
 *  Pins mapping def
 */
#define NEFF 10// Number of effects
#define N  7  // Number of LED
#define L1 2  // LED 1 (+) (with 220 ohms resistor)
#define L2 3  // LED 2 (+) (with 220 ohms resistor)
#define L3 4  // LED 3 (+) (with 220 ohms resistor)
#define L4 5  // LED 4 (+) (with 220 ohms resistor)
#define L5 6  // LED 4 (+) (with 220 ohms resistor)
#define L6 7  // LED 4 (+) (with 220 ohms resistor)
#define L7 8  // LED 4 (+) (with 220 ohms resistor) (A0)
#define Speaker A4 // Speaker (-) (with 100 ohms resistor) (12)
#define Button A5  // Button (-) (13)
//#define PowerCmd 7 // Auto Power Cmd bit ON/OFF
/***
 * Memory Leds and Effects maps
 */
PROGMEM unsigned char LEDmap[N] = {L1,L2,L3,L4,L5,L6,L7};
PROGMEM const unsigned char EFFECTmap0[] = {
  0,
  L1,L2,L3,L4,L5,L6,L7,0,
  0xFF
};
PROGMEM const unsigned char EFFECTmap1[] = {
  L1,L3,L5,L7,0,
  L2,L4,L6,0,
  0xFF
};
PROGMEM const unsigned char EFFECTmap2[] = {
  0,
  L7,0,
  L7,L6,0,
  L7,L6,L5,0,
  L7,L6,L5,L4,0,
  L7,L6,L5,L4,L3,0,
  L7,L6,L5,L4,L3,L2,0,
  L7,L6,L5,L4,L3,L2,L1,0,
  0xFF
};
PROGMEM const unsigned char EFFECTmap3[] = {
  0,
  L1,0,
  L1,L2,0,
  L1,L2,L3,0,
  L2,L3,L4,0,
  L3,L4,L5,0,
  L4,L5,L6,0,
  L5,L6,L7,0,
  L6,L7,0,
  L7,0,
  0,
  L7,0,
  L6,L7,0,
  L5,L6,L7,0,
  L4,L5,L6,0,
  L3,L4,L5,0,
  L2,L3,L4,0,
  L1,L2,L3,0,
  L1,L2,0,
  L1,0,
  0xFF
};
PROGMEM const unsigned char EFFECTmap4[] = {
  L1,0,
  L2,0,  
  L3,0,
  L4,0,
  L5,0,
  L6,0,
  L7,0,
  0xFF
};
PROGMEM const unsigned char EFFECTmap5[] = {
  L1,L2,L5,L6,0,
  L3,L4,L7,0,
  0xFF
};
PROGMEM const unsigned char EFFECTmap6[] = {
  L1,0,
  L1,L2,0,  
  L2,0,
  L2,L3,0,
  L3,0,
  L3,L4,0,
  L4,0,
  L4,L5,0,
  L5,0,
  L5,L6,0,
  L6,0,
  L6,L7,0,
  L7,0,
  L7,L1,0,
  0xFF
};
PROGMEM const unsigned char EFFECTmap7[] = {
  0,
  L4,L5,0,
  L3,L4,L5,L6,0,  
  L2,L3,L4,L5,L6,L7,0,
  L1,L2,L3,L4,L5,L6,L7,0,
  L2,L3,L4,L5,L6,L7,0,
  L1,L2,L3,L4,L5,L6,L7,0,
  L2,L3,L4,L5,L6,L7,0,
  L1,L2,L3,L4,L5,L6,L7,0,
  L2,L3,L4,L5,L6,L7,0,
  L3,L4,L5,L6,0,
  L4,L5,0,
  0xFF
};
PROGMEM const unsigned char EFFECTmap8[] = {
  0,
  L1,L2,L3,L4,L5,L6,L7,0,
  0,
  L1,L2,L3,L4,L5,L6,L7,0,
  0,
  L1,L2,L3,L4,L5,L6,L7,0,
  0,0,0,0,
  0xFF
};
PROGMEM const unsigned char EFFECTmap9[] = {
  0xFE,0,
  0xFE,0,
  0xFE,0,
  0xFE,0,
  0xFE,0,
  0xFE,0,
  0xFE,0,
  0xFF
};
PROGMEM const unsigned char *EFFECTSTable[NEFF] = {
  EFFECTmap0, EFFECTmap1, EFFECTmap2, EFFECTmap3, EFFECTmap4, EFFECTmap5, EFFECTmap6, EFFECTmap7, EFFECTmap8, EFFECTmap9
};
// -------------------------------------------------
// Copyright (c) 2022 HiBit <https://www.hibit.dev>
// -------------------------------------------------

PROGMEM int melody[] = {
  NOTE_AS4, 8, NOTE_AS4, 8, NOTE_AS4, 8,
  NOTE_F5,  2, NOTE_C6,  2,
  NOTE_AS5, 8, NOTE_A5,  8, NOTE_G5,  8, NOTE_F6, 2, NOTE_C6, 4,
  NOTE_AS5, 8, NOTE_A5,  8, NOTE_G5,  8, NOTE_F6, 2, NOTE_C6, 4,
  NOTE_AS5, 8, NOTE_A5,  8, NOTE_AS5,  8, NOTE_G5, 2, NOTE_C5, 8, NOTE_C5, 8, NOTE_C5, 8,
  NOTE_F5,  2, NOTE_C6,  2,
  NOTE_AS5, 8, NOTE_A5,  8, NOTE_G5,  8, NOTE_F6, 2, NOTE_C6, 4,

  NOTE_AS5, 8, NOTE_A5,  8, NOTE_G5,  8, NOTE_F6, 2, NOTE_C6, 4,
  NOTE_AS5, 8, NOTE_A5,  8, NOTE_AS5, 8, NOTE_G5, 2, NOTE_C5, 8, NOTE_C5, 16,
  NOTE_D5,  4, NOTE_D5,  8, NOTE_AS5, 8, NOTE_A5, 8, NOTE_G5, 8, NOTE_F5, 8,
  NOTE_F5,  8, NOTE_G5,  8, NOTE_A5,  8, NOTE_G5, 4, NOTE_D5, 8, NOTE_E5, 4, NOTE_C5, 8, NOTE_C5, 16,
  NOTE_D5,  4, NOTE_D5,  8, NOTE_AS5, 8, NOTE_A5, 8, NOTE_G5, 8, NOTE_F5, 8,

  NOTE_C6,  8, NOTE_G5, 16, NOTE_G5,  2, NOTE_C5, 8,
  NOTE_D5,  4, NOTE_D5,  8, NOTE_AS5, 8, NOTE_A5, 8, NOTE_G5, 8, NOTE_F5, 8,
  NOTE_F5,  8, NOTE_G5,  8, NOTE_A5,  8, NOTE_G5, 4, NOTE_D5, 8, NOTE_E5, 4, NOTE_C6, 8, NOTE_C6, 16,
  NOTE_F6,  4, NOTE_DS6, 8, NOTE_CS6, 4, NOTE_C6, 8, NOTE_AS5, 4, NOTE_GS5, 8, NOTE_G5, 4, NOTE_F5, 8,
  NOTE_C6,  1,
  0
};
/******
 * functions declaration
 */
void showEffect(const unsigned char *map, unsigned char repeat, unsigned int t);
void setAll(const unsigned char *ledmap, unsigned char nled, unsigned char value);

/******
 * setup
 */
void setup() {
  // Init Random value
  srand(12345);
  
  // Setup pins
  for (int n=0; n<N; n++) {
    pinMode(LEDmap[n], OUTPUT); // Set up LED PINS OUTPUT
    digitalWrite(LEDmap[n], LOW); // LED pins LOW (OFF)
  }
  
//  pinMode(Button, INPUT_PULLUP); // Set up button pins INPUT
  pinMode(Speaker, OUTPUT); // Speaker
  digitalWrite(Speaker, HIGH); // (OFF) High
  
#ifdef DEBUG
  // Serial port init
#if defined(ARDUINO_ARCH_8051)
  Serial89Init(); // Serial 89 init
#endif
  Serial.begin(9600); // Enable serial output so we can debug using the serial reader in the Arduino IDE
#endif
}
/******
 * loop
 */
void loop() {
    static unsigned char effect, repeat;
    static unsigned int t;
    
//  if ((digitalRead(Button)==LOW) && (buttonpressed==0)) { // if button pressed
//  }
    effect=random(0, NEFF-1); // Randomize n number ?
    repeat=random(1, 5); // Randomize repeat number ?
    t=random(25, 275); // Randomize delay time ?
    showEffect(EFFECTSTable[effect], repeat+300/t, t); // Random mode

/*
   unsigned int note;
    for (note=0;melody[note]>0;note+=2) {
      tone(Speaker, melody[note], 1000/melody[note+1]);
      delay(1000/melody[note+1]);
      noTone(Speaker);
    }
    */
   
}
/******
 * showEffect
 */
void showEffect(const unsigned char *effmap, unsigned char repeat, unsigned int t) {
  for (;repeat>0;repeat--) {
    int s=0; // Step
    while (effmap[s]!=0xFF) { // End of Effect ?
      setAll(LEDmap, N, LOW); // Clear all leds
      while (effmap[s]!=0) { // End of step (line) ?
#ifdef DEBUG
        Serial.println("S=%d map=%d", s, effmap[s]);
#endif
        if (effmap[s]!=0xFE) digitalWrite(effmap[s], HIGH); // LED pin HIGH (ON)
          else digitalWrite(LEDmap[random(0,N-1)], HIGH); // Randomize pin HIGH
        s++; // Next led
      }
      s++; // Next step
      delay(t);
    }
  }
}
/******
 * Set all
 */
 void setAll(const unsigned char *ledmap, unsigned char nled, unsigned char value) {
for (unsigned char n=0; n<nled; n++) {
    digitalWrite(ledmap[n], value); // LED pins LOW (OFF)
  }
}
